from socket import error as SocketError
from time import time, sleep

from adsopy import adso
adsotate = adso.Adso()

from crisucker import LinkPage, ArticleLoader, ArticlePage, load_mainpage_links

## annotate helper function ###################################################

def annotate(zhtext):
    """Use adsotate.py to convert Chinese text to annotated HTML"""
    annotation = []
    adso = None
    sleeptime = 2.0
    while adso is None:
        try:
            adso = adsotate.adsotate(zhtext.encode("utf-8"))
        except (SocketError, IOError):
            #failed, try again
            print "Couldn't connect to Adsotate, trying again in %s seconds." % sleeptime
            sleep(sleeptime) #so we don't hammer the adso server
            sleeptime += 2.0
            pass
    for (zh, pinyin, english) in adso:
        if type(pinyin) is unicode:
            unique_id = hash((zh, time()))
            zhtext = zhtext.replace(zh, """<span class="anno" hoverId="hover%s">%s</span>""" % (unique_id, zh))
            annotation.append("""<div id="hover%s" class="inlinePopup" style="display:none"><p class="zh">%s</p><p class="py">%s</p><p class="en">%s</p></div>""" % (unique_id, zh, pinyin, english))
    return zhtext, "\n".join(annotation)

## article to html ############################################################

HEADER = """<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/DTD/loose.dtd">
<html lang="zh">
    <head>
        <meta http-equiv="content-type" content="text/html;charset=utf-8">
        <title>CRI Online - %s</title>
        <link href="style.css" rel="stylesheet" type="text/css" />
        <script type="text/javascript" src="tooltip/yahoo.js"></script>
        <script type="text/javascript" src="tooltip/dom.js"></script>
        <script type="text/javascript" src="tooltip/event.js"></script>
        <script type="text/javascript" src="tooltip/animation.js"></script>
        <script type="text/javascript" src="tooltip/anno.js"></script>
    </head>
<body>
  <a name="top"></a>
  <h1>%s</h1>
  <p id="mmslink"><a href="%s">Listen</a> to this broadcast.</p>
"""

def page_header(title, mms_url):
   """Create a header for the generated page"""
   return HEADER % (title, title, mms_url)

def build_toc(article_titles):
   """Build a linked up table of contents for these articles"""
   out = []
   out.append("""  <ul id="toc">\n""")
   for title in article_titles:
      out.append("""    <li><a href="#%s">%s</a></li>\n""" % (title, title))
   out.append("""  </ul>\n""")
   return "".join(out)

def article_to_html(article_title, article_lines):
   """Annotate and convert article text back into HTML"""
   out = []
   out.append("""    <a name="%s"></a>\n""" % article_title)
   article_title, annotation = annotate(article_title)
   out.append("""    <h2>%s</h2>\n""" % article_title)
   out.append(annotation)
   for line in article_lines:
      line = line.strip()
      if line:
         line = line
         out.append("""      <p>%s</p>\n""" % line)
   return "".join(out)

def page_footer():
   """Close the page"""
   return """<p><a href="#top">Back to the top.</a></p>\n</body>\n</html>"""

## main #######################################################################

if __name__ == "__main__":
   links = []
   print "Loading main page links.",
   while len(links) == 0:
      print ".",
      links = load_mainpage_links() # grab all of the links from the main page
   print len(links), "!"
   #links = links[:20] #do the 20 most recent
   links = links[:1] #only do the most recent
   article_links = ArticleLoader(links) # grab all of the article links
   article_links.run()

   link_dict = article_links.link_dict
   for (title, link) in link_dict:
       print "-->", link
       all_article_text = ""
       mms = ""
       titles = []
       for (art_title, url) in link_dict[(title, link)]:
           #grab the actual articles
           if url.startswith("mms:"):
               print "MMS stream: %s" % url
               mms = url
           else:
               print url,
               titles.append(art_title)
               page = ArticlePage(url)
               while len(page.article) == 0:
                   print ".",
                   page.get()
               print len(page.article), "!"
               all_article_text += article_to_html(art_title, page.article)

       outname = link.split("/", 4)[-1].replace("/", "-")
       outfile = open("articles/%s" % outname, "w")
       outfile.write(page_header(title, mms).encode("utf-8"))
       outfile.write(build_toc(titles).encode("utf-8"))
       outfile.write(all_article_text.encode("utf-8"))
       outfile.write(page_footer().encode("utf-8"))
       outfile.close()
       print ""
