from HTMLParser import HTMLParser, HTMLParseError
from urllib import FancyURLopener
from urlparse import urlparse, urlunparse
import re

main_url = 'http://gb.chinabroadcast.cn/1321/more/4974/more4974.htm'
main_marker = 'a13'
date_re = re.compile(r'.*(2006.*)')
catch_all_re = re.compile(r'(.*)')

waiting = 0
title = 1
article = 2
done = 3

title_marker = 'title33'
article_marker = 'a63'

#   main_url (http://gb.chinabroadcast.cn/1321/more/4974/more4974.htm)
#   |--- article lists by day and time (e.g. http://gb.chinabroadcast.cn/1321/2006/11/10/542@1297615.htm)
#        |--- article text (e.g. http://gb.chinabroadcast.cn/1321/2006/11/10/542@1297617.htm)
#        |--- ...
#   |--- ...

# the process is:
# LinkPage (load_links) -> ArticleLoader (for each link) -> ArticlePage (for each article in each list)
#     main_url          -> articles by day and time      -> article text

## ArticlePage #############################################################

class ArticlePage(HTMLParser):
    """Parses an article page (e.g. "http://gb.chinabroadcast.cn/1321/2006/11/10/542@1297617.htm")"""
    def __init__(self, url):
        HTMLParser.__init__(self)
        self.url = url
        self.title = ''
        self.article = []
        self.articleText = ''
        self.state = waiting

    def get(self):
        file = FancyURLopener().open(self.url)
        for line in file:
            try:
                self.feed(line)
            except HTMLParseError, inst:
                print 'Parse error on line (%d:%d): %s' % (inst.lineno, inst.offset, inst.msg)
                self.reset()
            if self.state == done:
                break
        file.close()

    def onWaiting(self, tag, attrs):
        if tag == 'td':
            for name, value in attrs:
                if name == 'class':
                    if value == title_marker:
                        self.state = title
                    elif value == article_marker:
                        self.state = article

    def onArticle(self, tag):
        if tag == 'p':
            self.article.append(self.articleText)
            self.articleText = ''

    def handle_starttag(self, tag, attrs):
        if self.state == waiting:
            self.onWaiting(tag, attrs)
        elif self.state == article:
            self.onArticle(tag)

    def handle_endtag(self, tag):
        if tag == 'td':
            if self.state == title:
                self.state = waiting
            elif self.state == article:
                self.state = done

    def handle_data(self, data):
        if self.state == title:
            text = data.strip()
            if text:
                self.title = unicode(text, 'gb2312', 'ignore')
        elif self.state == article:
            text = unicode(data.strip(), 'gb2312', 'ignore')
            if text:
                self.articleText = self.articleText + text

## ArticleLoader ###########################################################

class ArticleLoader:
    """Parses the individual article links out of a list of links (e.g. [("title", "http://gb.chinabroadcast.cn/1321/2006/11/10/542@1297615.htm"), ...)]"""
    def __init__(self, links):
        self.links = links
        self.link_dict = {}

    def run(self):
        for title, link in self.links:
            page = LinkPage(link, article_marker)
            file = FancyURLopener().open(link)
            for line in file:
                try:
                    page.feed(line)
                except HTMLParseError, inst:
                    print 'Parse error on line (%d:%d): %s' % (inst.lineno, inst.offset, inst.msg)
                    page.reset()
                if page.done:
                    break
            file.close()
            self.link_dict[(title, link)] = page.links

## LinkPage ################################################################

class LinkPage(HTMLParser):
    """Parses the links out of a page"""
    def __init__(self, url, marker, match=catch_all_re):
        HTMLParser.__init__(self)
        self.url = url
        self.marker = marker
        self.re = match
        self.readingLinks = 0
        self.curLink = ''
        self.done = 0
        self.parsedURL = urlparse(self.url)
        self.links = []

    def checkForStart(self, tag, attrs):
        if tag == 'td':
            for name, value in attrs:
                if name == 'class' and value == self.marker:
                    self.readingLinks = 1

    def readLinks(self, tag, attrs):
        if tag == 'a':
            for name, value in attrs:
                if name == 'href':
                    url = urlparse(value)
                    scheme = url[0]
                    if scheme == '':
                        scheme = self.parsedURL[0]
                    netloc = url[1]
                    if netloc == '':
                        netloc = self.parsedURL[1]

                    parts = (scheme, netloc, url[2], '', '', '')
                    self.curLink = urlunparse(parts)

    def handle_starttag(self, tag, attrs):
        if self.readingLinks:
            self.readLinks(tag, attrs)
        else:
            self.checkForStart(tag, attrs)

    def handle_endtag(self, tag):
        if tag == 'table':
            if self.readingLinks:
                self.readingLinks = 0
                self.curLink = ''
                self.done = 1

    def handle_data(self, data):
        if self.curLink != '':
            match = self.re.match(data)
            if match and match.group(1) != '':
                pair = (unicode(match.group(1), 'gb2312', 'ignore'), self.curLink)
                self.links.append(pair)
                self.curLink = ''

## load links #################################################################

def load_mainpage_links():
    """Parse the links out of the main page of the site. (Start here)"""
    page = LinkPage(main_url, main_marker, date_re)
    file = FancyURLopener().open(main_url)
    for line in file:
        try:
            page.feed(line)
        except HTMLParseError, inst:
            print 'Parse error on line (%d:%d): %s' % (inst.lineno, inst.offset, inst.msg)
            page.reset()
        if page.done:
            break
    file.close()
    return page.links

